import os
import parsing
from lggAgent import LGGResolver
from classifier import Classifier


def mandatoryTask(pathToData):
    '''
    This method runs your implementation of LGGResolver on set of clauses in the file provided.
    
    :type pathToData: str 
    :rtype: tuple of Clause and Parser 
    '''
    parser = parsing.Parser(pathToData)
    clausesDataset = parser.parseClauses()

    resolver = LGGResolver(clausesDataset.variableFactory)

    for clause in clausesDataset.clauses:
        resolver.runLGG(clause)

    print('final clause learned by LGG:', resolver.getHypothesis())

    return resolver.getHypothesis(), parser


def optionalTask(clause, pathToInterpretationDataset, parser):
    '''
    This method runs evaluation of the optional task. It evaluates given clause on the dataset of interpretations. 
    
    :type clause: Clause
    :type pathToInterpretationDataset: str 
    :type parser: Parser 
    :rtype: None 
    '''
    interpretationsDataset = parsing.Parser(pathToInterpretationDataset, variableFactory=parser.variableFactory,
                                            predicateFactory=parser.predicateFactory).parseDatasetOfInterpretations()

    if len(interpretationsDataset.samples) < 1:
        print('ending since there is no sample in the dataset')
        return

    predictor = Classifier(clause)
    sat = 0
    for sample in interpretationsDataset.samples:
        prediction = predictor.predict(sample.interpretation)
        note = 'wrong'
        if prediction is sample.positiveClass:
            note = 'right'
            sat += 1
        print('prediction was', note, 'on sample', sample)

    print('accuracy of the clause is', float(sat) / len(interpretationsDataset.samples))


if __name__ == "__main__":
    pathToClausesDataset = os.sep.join([".", "data", "cores"])
    clause, parser = mandatoryTask(pathToClausesDataset)

    # set to True if you want to run the optional task
    if False:
        pathToInterpretationDataset = os.sep.join([".", "data", "coresInterpre"])
        optionalTask(clause, pathToInterpretationDataset, parser)
